<?php

namespace Drupal\eca_helper\Plugin\Action;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\eca\Service\YamlParser;
use Drupal\eca_form\Plugin\Action\FormFieldActionBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Exception\ParseException;

/**
 * Set access to a form field.
 *
 * @Action(
 *   id = "eca_helper_form_field_set_value",
 *   label = @Translation("ECA Helper: Form field set value"),
 *   description = @Translation("Set form field value."),
 *   type = "form"
 * )
 */
class FormFieldSetValue extends FormFieldActionBase {

  /**
   * The YAML parser.
   */
  protected YamlParser $yamlParser;

  /**
   * Whether to use form field value filters or not.
   *
   * @var bool
   */
  protected bool $useFilters = FALSE;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->setYamlParser($container->get('eca.service.yaml_parser'));
    return $instance;
  }

  /**
   * Set the YAML parser.
   *
   * @param \Drupal\eca\Service\YamlParser $yaml_parser
   *   The YAML parser.
   */
  public function setYamlParser(YamlParser $yaml_parser): void {
    $this->yamlParser = $yaml_parser;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'key' => '',
      'value' => '',
      'value_yaml' => FALSE,
      'array' => FALSE,
      'method' => 'append',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Element key'),
      '#description' => $this->t('The element key to set value for the form element. Example #title, #attributes.class'),
      '#default_value' => $this->configuration['key'],
      '#weight' => -49,
      '#eca_token_replacement' => TRUE,
    ];

    $form['value'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Value'),
      '#description' => $this->t('The default value to prepopulate.'),
      '#default_value' => $this->configuration['value'],
      '#weight' => -48,
      '#eca_token_replacement' => TRUE,
    ];

    $form['value_yaml'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Value in YAML'),
      '#description' => $this->t('Value in the YAML format.'),
      '#required' => FALSE,
      '#return_value' => 1,
      '#weight' => -48,
      '#default_value' => $this->configuration['value_yaml'],
    ];

    $form['array'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Set array value'),
      '#description' => $this->t('Set or append data to array value.'),
      '#required' => FALSE,
      '#return_value' => 1,
      '#weight' => -47,
      '#default_value' => $this->configuration['array'],
    ];

    $form['method'] = [
      '#type' => 'select',
      '#title' => $this->t('Method'),
      '#default_value' => $this->configuration['method'],
      '#description' => $this->t('The method to set value when use Set array value.'),
      '#weight' => -40,
      '#options' => [
        'set:clear' => $this->t('Set and clear previous value'),
        'set:empty' => $this->t('Set only when empty'),
        'append' => $this->t('Append to exist value'),
        'remove' => $this->t('Remove value instead of adding it'),
      ],
    ];
    $form = parent::buildConfigurationForm($form, $form_state);
    $ex_description = $this->t('Use <em>!!form</em> for the current form instead of form element.');
    $form['field_name']['#description'] = $form['field_name']['#description'] . ' ' . $ex_description;
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['key'] = $form_state->getValue('key');
    $this->configuration['value'] = $form_state->getValue('value');
    $this->configuration['value_yaml'] = $form_state->getValue('value_yaml');
    $this->configuration['array'] = $form_state->getValue('array');
    $this->configuration['method'] = $form_state->getValue('method');
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function &getTargetElement(): ?array {
    if ($this->configuration['field_name'] === '!!form') {
      $form = &$this->getCurrentForm();
      return $form;
    }
    return parent::getTargetElement();
  }

  /**
   * {@inheritdoc}
   */
  protected function doExecute(): void {
    if ($element = &$this->getTargetElement()) {
      if ($this->configuration['field_name'] !== '!!form') {
        $element = &$this->jumpToFirstFieldChild($element);
      }
      $key = $this->tokenService->getOrReplace($this->configuration['key']);
      $value = $this->tokenService->getOrReplace($this->configuration['value']);
      $array = $this->tokenService->getOrReplace($this->configuration['array']);
      $method = $this->tokenService->getOrReplace($this->configuration['method']);

      $value_yaml = $this->tokenService->getOrReplace($this->configuration['value_yaml']);
      if ($value_yaml) {
        try {
          $value = $this->yamlParser->parse($value);
        }
        catch (ParseException $e) {
          \Drupal::logger('eca')
            ->error('ECA Helper: Form field set value: Tried parsing value as YAML format, but parsing failed.');
          return;
        }
      }

      if ($key && is_string($key) && mb_strlen($key)) {
        if ($array) {
          $array_value = NestedArray::getValue($element, explode(".", $key)) ?? [];
          if (!is_array($array_value)) {
            $array_value = [$array_value];
          }
          if ($method === 'set:clear') {
            $array_value[] = $value;
          }
          if ($method === 'set:empty' && empty($array_value)) {
            $array_value = [$value];
          }
          if ($method === 'append') {
            $array_value[] = $value;
          }
          if ($method === 'remove') {
            $array_value = array_values(array_diff($array_value, is_array($value) ? $value : [$value]));
          }
          NestedArray::setValue($element, explode(".", $key), $array_value);
          return;
        }
        NestedArray::setValue($element, explode(".", $key), $value);
      }
    }
  }

}
